@@ -435,6 +435,8 @@ def floats(
     allow_nan=None,  # type: bool
     allow_infinity=None,  # type: bool
     width=64,  # type: int
+    exclude_min=False,  # type: bool
+    exclude_max=False,  # type: bool
 ):
     # type: (...) -> SearchStrategy[float]
     """Returns a strategy which generates floats.
@@ -456,11 +458,20 @@ def floats(
     Half-precision floats (``width=16``) are only supported on Python 3.6, or
     if :pypi:`Numpy` is installed.
 
+    The exclude_min and exclude_max argument can be used to generate numbers
+    from open or half-open intervals, by excluding the respective endpoints.
+    Attempting to exclude an endpoint which is None will raise an error;
+    use ``allow_infinity=False`` to generate finite floats.  You can however
+    use e.g. ``min_value=float("-inf"), exclude_min=True`` to exclude only
+    one infinite endpoint.
+
     Examples from this strategy have a complicated and hard to explain
     shrinking behaviour, but it tries to improve "human readability". Finite
     numbers will be preferred to infinity and infinity will be preferred to
     NaN.
     """
+    check_type(bool, exclude_min, "exclude_min")
+    check_type(bool, exclude_max, "exclude_max")
 
     if allow_nan is None:
         allow_nan = bool(min_value is None and max_value is None)
@@ -506,6 +517,15 @@ def floats(
             since="2018-10-10",
         )
 
+    if exclude_min:
+        if min_value is None:
+            raise InvalidArgument("Cannot exclude min_value=None")
+        min_value = next_up(min_value, width=width)
+    if exclude_max:
+        if max_value is None:
+            raise InvalidArgument("Cannot exclude max_value=None")
+        max_value = next_down(max_value, width=width)
+
     check_valid_interval(min_value, max_value, "min_value", "max_value")
     if min_value == float(u"-inf"):
         min_value = None
@@ -633,7 +653,7 @@ def sampled_from(elements):
 @overload
 def sampled_from(elements):
     # type: (Type[enum.Enum]) -> SearchStrategy[Any]
-    # `SearchStrategy[Enum]` is unreliable due to mataclass issues.
+    # `SearchStrategy[Enum]` is unreliable due to metaclass issues.
     pass  # pragma: no cover
 
 
@@ -655,6 +675,12 @@ def sampled_from(elements):
     """
     values = check_sample(elements, "sampled_from")
     if not values:
+        note_deprecation(
+            "sampled_from() with nothing to sample is deprecated and will be an "
+            "error in a future version.  It currently returns `st.nothing()`, "
+            "which if unexpected can make parts of a strategy silently vanish.",
+            since="2019-03-12",
+        )
         return nothing()
     if len(values) == 1:
         return just(values[0])
@@ -985,7 +1011,7 @@ def text(
     elif isinstance(alphabet, SearchStrategy):
         char_strategy = alphabet
     else:
-        char_strategy = sampled_from(list(alphabet))
+        char_strategy = sampled_from(list(alphabet)) if alphabet else nothing()
         if not isinstance(alphabet, abc.Sequence):
             raise InvalidArgument(
                 "alphabet must be an ordered sequence, or tests may be "
@@ -1146,6 +1172,7 @@ def builds(
     If the callable is a class defined with :pypi:`attrs`, missing required
     arguments will be inferred from the attribute on a best-effort basis,
     e.g. by checking :ref:`attrs standard validators <attrs:api_validators>`.
+    Dataclasses are handled natively by the inference from type hints.
 
     Examples from this strategy shrink by shrinking the argument values to
     the callable.
@@ -1575,7 +1602,7 @@ def decimals(
         special.append(Decimal("Infinity"))
     if allow_infinity or (allow_infinity is min_value is None):
         special.append(Decimal("-Infinity"))
-    return strat | sampled_from(special)
+    return strat | (sampled_from(special) if special else nothing())
 
 
 def recursive(
@@ -2140,5 +2167,5 @@ def emails():
     local_part = text(local_chars, min_size=1, max_size=64)
     # TODO: include dot-atoms, quoted strings, escaped chars, etc in local part
     return builds(u"{}@{}".format, local_part, domains()).filter(
-        lambda addr: len(addr) <= 255
+        lambda addr: len(addr) <= 254
     )
